@@ -18,13 +18,15 @@
 (**************************************************************************)
 
 open Printf
-open Benl_core
-open Benl_base
 open Tyxml.Html
+open Ben
+open Types
+open Core
+open Common
 
 let use_colors = ref false
 let output_file = ref None
-let input_source = ref Benl_types.NoSource
+let input_source = ref Ben.Types.NoSource
 let baseurl = ref "https://people.canonical.com/~ubuntu-archive/transitions/"
 
 type output_format = Text | Xhtml | Levels | Json
@@ -36,16 +38,29 @@ let output_format_from_string = function
   | "color" -> use_colors := true; Text
   | _ -> Levels
 
+let format_of_string = function
+  | "html" -> Xhtml
+  | "json" -> Json
+  | "txt" -> Text
+  | "levels" -> Levels
+  | s -> Printf.ksprintf invalid_arg "invalid format: %s" s
+
+let string_of_format = function
+  | Text -> "txt"
+  | Json -> "json"
+  | Xhtml -> "html"
+  | Levels -> "levels"
+
 let map_fun f l =
-  let fr = Benl_frontend.get_selected_frontend () in
-  if fr.Benl_frontend.name = "monitor" then
-    Benl_parallel.map ~level:(Benl_parallel.get_level ()) f l
+  let fr = Frontend.get_selected_frontend () in
+  if fr.Frontend.name = "monitor" then
+    Parallel.map ~level:(Parallel.get_level ()) f l
   else
     List.map f l
 
 let get_config config key =
   try StringMap.find key config
-  with Not_found -> Benl_error.raise (Benl_error.Missing_configuration_item key)
+  with Not_found -> Error.raise (Error.Missing_configuration_item key)
 
 let is_affected config =
   lazy (Query.of_expr (get_config config "is_affected"))
@@ -54,9 +69,9 @@ let is_good config =
 let is_bad config =
   lazy (Query.of_expr (get_config config "is_bad"))
 
-open Benl_modules
+open Modules
 open Marshallable
-open Benl_data
+open Data
 
 let format_arch x =
   let f = match x with
@@ -75,16 +90,16 @@ let format_arch x =
     else f
   in f
 
-let ben_webpage = "https://ben.debian.net"
+let ben_webpage = "https://debian.pages.debian.net/ben/"
 
-let print_dep_line src deps =
-  printf "%s:" !!!src;
-  S.iter (fun dep -> printf " %s" !!!dep) deps;
-  printf "\n%!"
+let print_dep_line fmt src deps =
+  Format.fprintf fmt "%s:" !!!src;
+  S.iter (fun dep -> Format.fprintf fmt " %s" !!!dep) deps;
+  Format.fprintf fmt "\n%!"
 
 let spec = Arg.align [
-  "--run-debcheck", Arg.Set Benl_data.run_debcheck, " Run debcheck to register installability information";
-  "--use-projectb", Arg.Set Benl_data.use_projectb, " Get package lists from Projectb database";
+  "--run-debcheck", Arg.Set Data.run_debcheck, " Run debcheck to register installability information";
+  "--use-projectb", Arg.Set Data.use_projectb, " Get package lists from Projectb database";
   "--output"      , Arg.String (fun filename ->
     output_file := Some filename
   )
@@ -102,25 +117,25 @@ let spec = Arg.align [
   )
                                                   , " Format of output";
   "-stdin"        , Arg.Unit (fun () ->
-    input_source := Benl_types.Stdin
+    input_source := Types.Stdin
   )
                                                   , " Use stdin to read the input file";
   "--template"    , Arg.String (fun template ->
-    Benl_templates.load_template template)
+    Templates.load_template template)
                                                   , " Select an HTML template";
 ]
 
 let check_media_dir base =
   let mediad = base // "media" in
   if not (Sys.file_exists mediad) then
-    Unix.symlink !Benl_clflags.media_dir mediad
+    Unix.symlink !Clflags.media_dir mediad
   else
     match (Unix.stat mediad).Unix.st_kind with
     | Unix.S_LNK ->
         let target = Unix.readlink mediad in
-        if target != !Benl_clflags.media_dir then begin
+        if target != !Clflags.media_dir then begin
           Unix.unlink mediad;
-          Unix.symlink !Benl_clflags.media_dir mediad
+          Unix.symlink !Clflags.media_dir mediad
         end
     | _ -> ()
 
@@ -149,17 +164,17 @@ let combine_states state1 state2 =
 
 let archs_list config =
   let expr_l = function
-    | Benl_types.EList l -> l
+    | Types.EList l -> l
     | _ -> assert false
   in
-  let release_archs_list = expr_l (Benl_clflags.get_config config "architectures") in
+  let release_archs_list = expr_l (Clflags.get_config config "architectures") in
   let ignored_archs_list =
     try
-      expr_l (Benl_clflags.get_config config "ignored")
+      expr_l (Clflags.get_config config "ignored")
     with _ -> []
   in
-  let archs_list = Benl_core.uniq (release_archs_list @ ignored_archs_list) in
-  List.sort Pervasives.compare archs_list
+  let archs_list = Core.uniq (release_archs_list @ ignored_archs_list) in
+  List.sort Stdlib.compare archs_list
 
 let compute_monitor_data config sources binaries rounds =
   map_fun begin fun xs ->
@@ -169,7 +184,7 @@ let compute_monitor_data config sources binaries rounds =
       let src_name = Package.get "package" src in
       let states =
         List.map begin function
-        | Benl_types.EString arch ->
+        | Types.EString arch ->
           (* FIXME: indexing by name+arch is not a good idea after all *)
           arch, PAMap.fold (fun (_, arch') pkg (reasons, accu) ->
             if arch' = arch && Package.get "source" pkg = src_name
@@ -192,14 +207,28 @@ let compute_monitor_data config sources binaries rounds =
     end packages
   end rounds
 
-let print_text_monitor config sources binaries rounds =
-  let monitor_data = compute_monitor_data config sources binaries rounds in
+type transition_data = {
+    config : Types.expr Core.StringMap.t;
+    monitor_data :
+      ([ `source ] Package.t *
+         (string * (string * status)) list)
+        list list;
+    sources : ([ `source ], [ `source ] Package.t) Package.Map.t;
+    binaries : [ `binary ] Package.t Modules.PAMap.t;
+    rounds : [ `source ] Package.Name.t list list;
+    dep_graph : ([ `source ], [ `source ] Package.Set.t) Package.Map.t;
+    all : int;
+    bad : int;
+    packages : [ `source ] Data.S.t;
+  }
+
+let print_text_monitor fmt {config; monitor_data; sources; rounds; _} =
   let nmax = M.fold begin fun src _ accu ->
     let n = String.length !!!src in
     if n > accu then n else accu
   end sources 0 in
   let architectures =
-    List.map (Benl_frontend.to_string "architectures") (archs_list config)
+    List.map (Frontend.to_string "architectures") (archs_list config)
   in
   let width =
     String.length (String.concat "   " architectures)+6+nmax
@@ -218,7 +247,7 @@ let print_text_monitor config sources binaries rounds =
     Scanf.format_from_string (Buffer.contents buf) "%d"
   in
   List.iteri begin fun i xs ->
-    printf header_fmt i;
+    Format.fprintf fmt header_fmt i;
     List.iter begin fun (src, states) ->
       let in_testing =
         try not (Package.get "is-in-testing" src = "no")
@@ -226,17 +255,16 @@ let print_text_monitor config sources binaries rounds =
       in
       let sname = Package.get "package" src in
       let sname = if in_testing then sname else "_"^sname in
-      printf "%*s:" (nmax+3) sname;
+      Format.fprintf fmt "%*s:" (nmax+3) sname;
       List.iter begin fun (arch, (_, state)) ->
-        printf " %s" (format_arch state arch)
+        Format.fprintf fmt " %s" (format_arch state arch)
       end states;
-      printf "\n";
+      Format.fprintf fmt "\n";
     end xs;
-    printf "\n"
+    Format.fprintf fmt "\n"
   end monitor_data
 
-let print_json_monitor config sources binaries rounds =
-  let monitor_data = compute_monitor_data config sources binaries rounds in
+let print_json_monitor fmt {monitor_data; _} =
   let levels = List.mapi begin fun i xs ->
     let srcs = List.map begin fun (src, states) ->
       let in_testing =
@@ -252,12 +280,12 @@ let print_json_monitor config sources binaries rounds =
     end xs in
     Printf.sprintf "  \"%d\" : {\n%s\n  }" i (String.concat ",\n" srcs)
   end monitor_data in
-  Printf.printf "{\n%s\n}" (String.concat ",\n" levels)
+  Format.fprintf fmt "{\n%s\n}" (String.concat ",\n" levels)
 
 let a_link url text =
-  a ~a:[a_href (uri_of_string url)] [pcdata text]
+  a ~a:[a_href (uri_of_string url)] [txt text]
 
-let escape = Netencoding.Url.encode
+let escape x = Uri.pct_encode ~component:`Fragment x
 
 let pts t src =
   a_link (t.Template.pts ~src:(escape src)) src
@@ -279,18 +307,19 @@ let changelog t ver dir src =
   small [ a_link ( t.Template.changelog ~letter:dir ~src ~ver) ver ]
 
 let generated_on_text () =
-  [ pcdata "Page generated by ";
-    a_link ben_webpage "Ben ";
-    pcdata Benl_version.version;
-    pcdata (Printf.sprintf " on %s" (Benl_core.get_rfc2822_date ()));
-    pcdata "; bzr branch ";
+  [ txt "Page generated by ";
+    a_link ben_webpage "Ben";
+    txt " ";
+    txt Version.version;
+    txt (Printf.sprintf " on %s" (Core.get_rfc2822_date ()));
+    txt "; bzr branch ";
     a_link "https://code.launchpad.net/+branch/ubuntu-transition-tracker" "lp:ubuntu-transition-tracker"
   ]
 
 let overrall_state l =
   let _ (* ignored_archs *), release_archs =
     List.partition
-      (fun (arch,_) -> List.mem arch !Benl_base.ignored_architectures)
+      (fun (arch,_) -> List.mem arch !Clflags.ignored_architectures)
       l in
   if List.for_all (fun (_,(_,status)) -> status = Unknown) release_archs then
     Unknown
@@ -320,9 +349,9 @@ let generate_stats monitor_data =
             all, bad, packages
           in
           match overrall_state with
-          | Outdated when (not !Benl_data.use_projectb || is_in_testing) ->
+          | Outdated when (not !Data.use_projectb || is_in_testing) ->
               return (all+1) (bad+1)
-          | Partial when (not !Benl_data.use_projectb || is_in_testing) ->
+          | Partial when (not !Data.use_projectb || is_in_testing) ->
               return (all+1) (bad+1)
           | Up_to_date ->
               return (all+1) bad
@@ -346,13 +375,13 @@ let cut_head text head =
   with _ -> text
 
 let beautify_text =
-  let r_link = Pcre.regexp "#[0-9]{4,}|[a-z]{3,}://[^\\s><]+|<[^\\s><]+@[^\\s><]+>|[Pp][Tt][Ss]:[a-z0-9+\\-\\.]+|[Bb][Uu][Ii][Ll][Dd][Dd]:[a-z0-9+\\-\\.]+" in
+  let r_link = Re.Pcre.regexp "#[0-9]{4,}|[a-z]{3,}://[^\\s><]+|<[^\\s><]+@[^\\s><]+>|[Pp][Tt][Ss]:[a-z0-9+\\-\\.]+|[Bb][Uu][Ii][Ll][Dd][Dd]:[a-z0-9+\\-\\.]+" in
   fun tpl text ->
-    let t = Pcre.full_split ~rex:r_link text in
+    let t = Re.Pcre.full_split ~rex:r_link text in
     List.map
       (function
-        | Pcre.Text s -> pcdata s
-        | Pcre.Delim s ->
+        | Re.Pcre.Text s -> txt s
+        | Re.Pcre.Delim s ->
           let l = String.lowercase_ascii s in
           if s.[0] = '#' then
             let ss = String.sub s 1 (String.length s -1) in
@@ -372,7 +401,7 @@ let beautify_text =
             a_link link s
           else
             a_link s s
-        | Pcre.Group _ | Pcre.NoGroup -> (* Ignore this case *) pcdata ""
+        | Re.Pcre.Group _ | Re.Pcre.NoGroup -> (* Ignore this case *) txt ""
       )
       t
 
@@ -394,7 +423,7 @@ let compute_transition_data data config =
   let rounds, sources, binaries, dep_graph = compute_graph data config in
   let monitor_data = compute_monitor_data config sources binaries rounds in
   let all, bad, packages = generate_stats monitor_data in
-  monitor_data, sources, binaries, dep_graph, all, bad, packages
+  {config; monitor_data; sources; binaries; rounds; dep_graph; all; bad; packages}
 
 let has_testing_data monitor_data =
   match monitor_data with
@@ -405,7 +434,8 @@ let has_testing_data monitor_data =
      with Not_found -> false)
   | _ -> false
 
-let print_html_monitor config template monitor_data binaries dep_graph packages has_testing_data extra =
+let print_html_monitor template {config; monitor_data; binaries; dep_graph; packages; _} extra =
+  let has_testing_data = has_testing_data monitor_data in
   let affected = packages in
   let mytitle =
     try
@@ -434,42 +464,47 @@ let print_html_monitor config template monitor_data binaries dep_graph packages
 
     script
       ~a:[a_src (uri_of_string "media/jquery.min.js")]
-      (pcdata "");
+      (txt "");
 
     script
       ~a:[a_src (uri_of_string ("media/script.js"))]
-      (pcdata "");
+      (txt "");
   ] in
   let hbody table = [
-    b [ pcdata "Parameters:" ];
+    b [ txt "Parameters:" ];
     ul~a:[ a_class ["parameters"] ]
-      [li [ small [ b [ pcdata "Affected: " ]; pcdata is_affected ] ];
-       li [ small [ b [ pcdata "Good: " ]; pcdata is_good ] ];
-       li [ small [ b [ pcdata "Bad: " ]; pcdata is_bad ] ];
+      [li [ small [ b [ txt "Affected: " ]; txt is_affected ] ];
+       li [ small [ b [ txt "Good: " ]; txt is_good ] ];
+       li [ small [ b [ txt "Bad: " ]; txt is_bad ] ];
       ];
     if String.length notes = 0 then
       div [ ]
     else
       div ~a:[ a_class ["parameters"] ]
-        [ small [ b [ pcdata "Notes: " ] ];
+        [ small [ b [ txt "Notes: " ] ];
           pre ( beautify_text template notes ) ]
     ;
     div
       [
-        pcdata "Filter by status: ";
+        txt "Filter by status: ";
         input ~a:[a_input_type `Checkbox; a_id "good"] ();
-        pcdata "good ";
+        label ~a:[a_label_for "good"] [txt "good"];
+        txt " ";
         input ~a:[a_input_type `Checkbox; a_checked (); a_id "partial"] ();
-        pcdata "partial ";
+        label ~a:[a_label_for "partial"] [txt "partial"];
+        txt " ";
         input ~a:[a_input_type `Checkbox; a_checked (); a_id "bad"] ();
-        pcdata "bad ";
-        input ~a:[a_input_type `Checkbox; a_checked (); a_id "unknown"] (); pcdata "unknown";
+        label ~a:[a_label_for "bad"] [txt "bad"];
+        txt " ";
+        input ~a:[a_input_type `Checkbox; a_checked (); a_id "unknown"] ();
+        label ~a:[a_label_for "unknown"] [txt "unknown"];
+        txt " ";
         span ~a:[a_id "count"] [];
       ];
     div (if has_testing_data
       then [
         input ~a:[a_input_type `Checkbox; a_id "notintesting"] ();
-        pcdata "ignore packages that are not in testing";
+        label ~a:[a_label_for "notintesting"] [txt "ignore packages that are not in testing"];
       ]
       else []
     );
@@ -483,10 +518,10 @@ let print_html_monitor config template monitor_data binaries dep_graph packages
     | "powerpc" -> "ppc"
     | x -> x in
   let archs_columns = List.map begin function
-    | Benl_types.EString arch when (List.mem arch !Benl_base.ignored_architectures) ->
-      th [ small [ i [ pcdata (abrege arch) ] ] ]
-    | Benl_types.EString arch ->
-      th [ small [ pcdata (abrege arch) ] ]
+    | Types.EString arch when (List.mem arch !Clflags.ignored_architectures) ->
+      th [ small [ i [ txt (abrege arch) ] ] ]
+    | Types.EString arch ->
+      th [ small [ txt (abrege arch) ] ]
     | _ -> assert false
   end (archs_list config) in
   let archs_columns round header =
@@ -505,7 +540,7 @@ let print_html_monitor config template monitor_data binaries dep_graph packages
                 let pkg = PAMap.find (bin, arch) binaries in
                 Package.get "multi-arch" pkg = "same"
               with Not_found -> false
-            ) (List.map (Benl_frontend.to_string "architectures") (archs_list config))
+            ) (List.map (Frontend.to_string "architectures") (archs_list config))
           ) (Package.binaries source)
         in
         let has_additional_text, additional_texts =
@@ -522,9 +557,9 @@ let print_html_monitor config template monitor_data binaries dep_graph packages
         let additional_text = String.concat ", " additional_texts in
         let additional_text_html =
           if has_additional_text then [
-            pcdata " [";
-            small [pcdata additional_text];
-            pcdata "]";
+            txt " [";
+            small [txt additional_text];
+            txt "]";
           ] else []
         in
         let in_testing =
@@ -549,7 +584,7 @@ let print_html_monitor config template monitor_data binaries dep_graph packages
         let sources = src::sources in
         let overrall_state = [ class_of_status (overrall_state states) ] in
         let src_text =
-          (pts template src)::(if in_testing then [] else [pcdata " (sid only)"])
+          (pts template src)::(if in_testing then [] else [txt " (sid only)"])
         in
         let acc =
           tr ([ td ~a:[ a_class ("srcname" :: (overrall_state @ classes));
@@ -558,16 +593,16 @@ let print_html_monitor config template monitor_data binaries dep_graph packages
                       ]
                    src_text;
                 td ~a:[ a_class ["src"] ]
-                   ([ pcdata "[";
+                   ([ txt "[";
                      buildd template (escape src) version;
-                     pcdata "] (";
+                     txt "] (";
                      changelog template (sprintf "%s" version) directory (escape src);
-                     pcdata ")" ] @ additional_text_html)
+                     txt ")" ] @ additional_text_html)
               ]
               @
                 (List.map begin fun (_, (reasons, state)) ->
-                                (td ~a:[ a_class [ Benl_base.class_of_status state ]; a_title reasons ]
-                                    [ small [ pcdata (Benl_base.string_of_status state) ] ])
+                                (td ~a:[ a_class [ class_of_status state ]; a_title reasons ]
+                                    [ small [ txt (string_of_status state) ] ])
                           end states)
              )
              :: acc
@@ -579,13 +614,13 @@ let print_html_monitor config template monitor_data binaries dep_graph packages
         let buildd_link = buildds template escaped_names in
         let rc_bugs_link = rc_bugs template escaped_names in
         [
-          pcdata " (";
-          buildd_link; pcdata " "; rc_bugs_link;
-          pcdata ")"
+          txt " (";
+          buildd_link; txt " "; rc_bugs_link;
+          txt ")"
         ]
         with Exit -> []
       in
-      let title = pcdata (sprintf "Dependency level %d" (i+1)) in
+      let title = txt (sprintf "Dependency level %d" (i+1)) in
       let tr_cols = archs_columns
                       i
                       (th ~a:[a_colspan 2; a_class ["level"]] (title::column_arg))
@@ -603,7 +638,7 @@ let print_html_monitor config template monitor_data binaries dep_graph packages
   in
   let subtitle =
     [a_link (Filename.concat !baseurl "index.html") "Transitions";
-     pcdata (Printf.sprintf " → %s" mytitle)
+     txt (Printf.sprintf " → %s" mytitle)
     ] in
   let html = template.Template.page
     ~title:page_title
@@ -614,63 +649,60 @@ let print_html_monitor config template monitor_data binaries dep_graph packages
   in
   html
 
-let print_dependency_levels dep_graph rounds =
+let print_dependency_levels fmt {dep_graph; rounds; _} =
   List.iteri begin fun i xs ->
-    printf "===[ Dependency level %d ]=====\n" i;
+    Format.fprintf fmt "===[ Dependency level %d ]=====\n" i;
     let packages = List.sort (fun x y -> compare !!!x !!!y) xs in
     List.iter begin fun src ->
       let deps = M.find src dep_graph in
-      print_dep_line src deps
+      print_dep_line fmt src deps
     end packages
   end rounds
 
+let print_monitor format fmt transition_data =
+  match format with
+  | Levels -> print_dependency_levels fmt transition_data
+  | Text -> print_text_monitor fmt transition_data
+  | Json -> print_json_monitor fmt transition_data
+  | Xhtml ->
+     let template = Templates.get_registered_template () in
+     let output = print_html_monitor template transition_data None in
+     Format.fprintf fmt "%a\n%!" (Tyxml.Html.pp ()) output
+
 let main _ =
   let config = match !input_source with
-    | Benl_types.NoSource -> Benl_error.raise Benl_error.Missing_configuration_file
-    | _ as source -> Benl_frontend.read_config ~multi:true source
+    | Types.NoSource -> Error.raise Error.Missing_configuration_file
+    | _ as source -> Frontend.read_config ~multi:true source
   in
-  let archs_list = Benl_frontend.to_string_l
+  let archs_list = Frontend.to_string_l
     "architectures"
-    (Benl_clflags.get_config config "architectures")
+    (Clflags.get_config config "architectures")
   in
-  let data = Benl_data.load_cache archs_list in
-  let rounds, sources, binaries, dep_graph =
-    compute_graph data config in
-  match !output_format with
-    | Levels -> print_dependency_levels dep_graph rounds
-    | Text -> print_text_monitor config sources binaries rounds
-    | Json -> print_json_monitor config sources binaries rounds
-    | Xhtml ->
-      let template = Benl_templates.get_registered_template () in
-      let monitor_data = compute_monitor_data config sources binaries rounds in
-      let _, _, packages = generate_stats monitor_data in
-      let has_testing_data = has_testing_data monitor_data in
-      let output =
-        print_html_monitor
-          config
-          template
-          monitor_data
-          binaries
-          dep_graph
-          packages
-          has_testing_data
-          None
-      in
-      match !output_file with
-      | None ->
-          Format.printf "%a" (Tyxml.Html.pp ()) output;
-          print_newline ()
-      | Some file ->
-          check_media_dir (Filename.basename file);
-          Benl_utils.dump_xhtml_to_file file output
+  let data = Data.load_cache archs_list in
+  let transition_data = compute_transition_data data config in
+  let fmt, close =
+    match !output_file with
+    | None -> Format.std_formatter, (fun () -> ())
+    | Some file ->
+       let oc = open_out file in
+       Format.formatter_of_out_channel oc,
+       (fun () -> close_out oc)
+  in
+  let () =
+    match !output_format, !output_file with
+    | Xhtml, Some file -> check_media_dir (Filename.basename file)
+    | _, _ -> ()
+  in
+  print_monitor !output_format fmt transition_data;
+  close ()
 
 let anon_fun file =
-  if Benl_core.ends_with file ".ben" then
-    input_source := Benl_types.File file
+  if Core.ends_with file ".ben" then
+    input_source := Types.File file
 
 let frontend = {
-  Benl_frontend.name = "monitor";
-  Benl_frontend.main = main;
-  Benl_frontend.anon_fun = anon_fun;
-  Benl_frontend.help = spec;
+  Frontend.name = "monitor";
+  Frontend.main = main;
+  Frontend.anon_fun = anon_fun;
+  Frontend.help = spec;
 }
