@@ -17,69 +17,73 @@
 (*  <http://www.gnu.org/licenses/>.                                       *)
 (**************************************************************************)
 
-open Tyxml.Html
 open Printf
-open Benl_core
-open Benl_clflags
-open Benl_utils
-open Benl_error
+open Tyxml.Html
+open Ben
+open Core
+open Clflags
+open Utils
+open Error
 
 let ($) f x = f x
 
 let base = ref "."
 let config_dir = ref "config"
-let global_config = ref (FilePath.concat !config_dir "global.conf")
 let lock = ref "ben.lock"
 let clean = ref true
 let tconfig = ref None
+let output_formats = ref [Ben_monitor.Xhtml]
 let redownload = ref true
 
-open Benl_types
-open Benl_modules
-open Benl_frontend
+open Types
+open Modules
+open Frontend
 
 let eval x =
   Query.eval
     `source
-    (Package.of_assoc `source Benl_core.StringMap.empty)
+    (Package.of_assoc `source Core.StringMap.empty)
     (Query.of_expr x)
 
-let read_global_config () =
-  if Sys.file_exists !global_config then begin
-    let config = Benl_utils.parse_config_file !global_config in
+let read_global_config file =
+  if Sys.file_exists file then begin
+    let config = Utils.parse_config_file file in
     StringMap.iter (fun key value -> match (key, value) with
         | "architectures", archs ->
-          Benl_base.debian_architectures := to_string_l "architectures" archs;
-          Benl_clflags.architectures := to_string_l "architectures" archs
+          Clflags.debian_architectures := to_string_l "architectures" archs
         | "ignored", archs ->
-          Benl_base.ignored_architectures := to_string_l "ignored" archs
+          Clflags.ignored_architectures := to_string_l "ignored" archs
         | "suite", (EString suite) ->
-          Benl_clflags.suite := suite
+          Clflags.suite := suite
         | "areas", areas ->
-          Benl_clflags.areas := to_string_l "areas" areas
+          Clflags.areas := to_string_l "areas" areas
         | "base", (EString path) ->
           base := path
         | "config-dir", (EString path) ->
           config_dir := path
         | "cache-dir", (EString dir) ->
-          Benl_clflags.cache_dir := dir
+          Clflags.cache_dir := dir
         | "mirror-binaries", (EString mirror) ->
-          Benl_clflags.mirror_binaries := mirror
+          Clflags.mirror_binaries := mirror
         | "mirror-sources", (EString mirror) ->
-          Benl_clflags.mirror_sources := mirror
+          Clflags.mirror_sources := mirror
         | "mirror", (EString mirror) ->
-          Benl_clflags.mirror_sources := mirror;
-          Benl_clflags.mirror_binaries := mirror
+          Clflags.mirror_sources := mirror;
+          Clflags.mirror_binaries := mirror
         | "use-cache", expr ->
-          Benl_clflags.use_cache := eval expr
+          Clflags.use_cache := eval expr
         | "run-debcheck", expr ->
-          Benl_data.run_debcheck := eval expr
+          Data.run_debcheck := eval expr
         | "use-projectb", expr ->
-          Benl_data.use_projectb := eval expr
+          Data.use_projectb := eval expr
         | "base-url", (EString url) ->
           Ben_monitor.baseurl := url
         | "template", (EString template) ->
-          Benl_templates.load_template template;
+          Templates.load_template template;
+        | "output-formats", formats ->
+           output_formats :=
+             to_string_l "output-formats" formats
+             |> List.map Ben_monitor.format_of_string
         | "redownload", Efalse ->
           redownload := false
         | item, _ ->
@@ -94,16 +98,17 @@ let lockf () =
 let spec = Arg.align [
   "--config-dir" , Arg.Set_string config_dir, " Specify tracker's global configuration directory";
   "-cd"          , Arg.Set_string config_dir, " ";
-  "--global-conf", Arg.Set_string global_config, " Specify tracker's global configuration file";
-  "-g"           , Arg.Set_string global_config, " ";
+  "--global-conf", Arg.String read_global_config, " Specify tracker's global configuration file";
+  "-g"           , Arg.String read_global_config, " ";
   "--transition" , Arg.String (fun t -> tconfig := Some t), " Specify selected transition";
   "-t"           , Arg.String (fun t -> tconfig := Some t), " ";
-  "--update"     , Arg.Set Benl_clflags.update, " Do update the cache file";
-  "-u"           , Arg.Set Benl_clflags.update, " ";
+  "--update"     , Arg.Set Clflags.update, " Do update the cache file";
+  "-u"           , Arg.Set Clflags.update, " ";
   "--base"       , Arg.Set_string base, " Specify tracker's base directory";
   "-b"           , Arg.Set_string base, " ";
-  "--use-projectb", Arg.Set Benl_data.use_projectb, " Use Projectb to fetch information about distributions";
-  "--template"   , Arg.String (fun t -> Benl_templates.load_template t), " Specify template to use";
+  "--run-debcheck", Arg.Set Data.run_debcheck, " Run debcheck to register installability information";
+  "--use-projectb", Arg.Set Data.use_projectb, " Use Projectb to fetch information about distributions";
+  "--template"   , Arg.String (fun t -> Templates.load_template t), " Specify template to use";
   "--no-clean"   , Arg.Clear clean, " Do not clean HTML directory before exiting";
 ]
 
@@ -136,7 +141,7 @@ let profiles_desc = [
 
 open FileUtil
 
-let p = Benl_clflags.progress
+let p = Clflags.progress
 
 let is_packages_file name =
   try
@@ -152,22 +157,22 @@ let clear_cache () =
   rm ~force:Force pkgs
 
 let update_test () =
-  let cachef = Benl_clflags.get_cache_file () in
-     !Benl_clflags.update
-  || (!Benl_clflags.use_cache && test (Not Exists) cachef)
+  let cachef = Clflags.get_cache_file () in
+     !Clflags.update
+  || (!Clflags.use_cache && test (Not Exists) cachef)
 
 let read_cache architectures =
   if update_test () then begin
-    if !redownload && not !Benl_data.use_projectb then begin
+    if !redownload && not !Data.use_projectb then begin
       clear_cache ();
       Ben_download.download_all architectures;
     end;
-    Benl_data.generate_cache
-      (Benl_clflags.get_cache_file ())
+    Data.generate_cache
+      (Clflags.get_cache_file ())
       architectures
   end
   else
-    Benl_data.load_cache architectures
+    Data.load_cache architectures
 
 let profile_of_file file =
   try
@@ -178,20 +183,18 @@ let read_transition_config file =
   let (!!) = Filename.basename in
   let transition = FilePath.chop_extension !!file in
   (* Read a .ben file *)
-  transition, (Benl_frontend.read_ben_file file)
+  transition, (Frontend.read_ben_file file)
+
+let get_export config =
+  try Clflags.get_config config "export" = Types.Etrue
+  with _ -> true
 
 let get_transition_data data name config =
   try
     let transition_data = Ben_monitor.compute_transition_data data config in
-    let monitor_data, _, _, _, _, _, _ = transition_data in
-    let has_testing_data = Ben_monitor.has_testing_data monitor_data in
-    let export =
-      try
-        Benl_clflags.get_config config "export" = Benl_types.Etrue
-      with _ -> true in
-    Some (export, transition_data, has_testing_data)
+    Some transition_data
   with e ->
-    Benl_error.warn_exn ("Failed to process transition " ^ name) e;
+    Error.warn_exn ("Failed to process transition " ^ name) e;
     None
 
 module SMap = Map.Make(String)
@@ -208,8 +211,8 @@ let smerge _ v1 v2 = match v1, v2 with
   | None   , Some v2 -> Some v2
   | _                -> None
 
-let html_path_t name =
-  Filename.concat "html" (Printf.sprintf "%s.html" name)
+let format_path_t format name =
+  Filename.concat format (Printf.sprintf "%s.%s" name format)
 
 let print_html_collisions (hits : (SMap.key * SMap.key list) list) =
   let hits = List.fold_left
@@ -223,7 +226,7 @@ let print_html_collisions (hits : (SMap.key * SMap.key list) list) =
     hits
   in
   div ~a:[ a_id "collisions" ] [
-    b [ pcdata "Collisions:" ];
+    b [ txt "Collisions:" ];
     let hits =
       SMap.fold
         (fun transition packages list ->
@@ -231,8 +234,8 @@ let print_html_collisions (hits : (SMap.key * SMap.key list) list) =
             li [ Ben_monitor.a_link
                    (transition ^ ".html")
                    transition;
-                 pcdata " through ";
-                 pcdata (String.concat ", " packages)
+                 txt " through ";
+                 txt (String.concat ", " packages)
                ]
           in
           elt :: list
@@ -241,13 +244,11 @@ let print_html_collisions (hits : (SMap.key * SMap.key list) list) =
         []
     in
     match List.rev hits with
-    | [] -> pcdata "(none)"
+    | [] -> txt "(none)"
     | h::l -> ul (h::l)
   ]
 
-let print_html_monitor config template file transition_data has_testing_data collisions =
-  let monitor_data, _, binaries, dep_graph, _, _, packages = transition_data in
-  let ($) = Filename.concat in
+let print_html_monitor fmt template file transition_data collisions =
   let (!!) = Filename.basename in
   let collisions_div =
     try
@@ -258,30 +259,44 @@ let print_html_monitor config template file transition_data has_testing_data col
   in
   let output =
     Ben_monitor.print_html_monitor
-      config
       template
-      monitor_data
-      binaries
-      dep_graph
-      packages
-      has_testing_data
+      transition_data
       collisions_div
   in
-  let htmlp = html_path_t !!file in
+  Format.fprintf fmt "%a\n%!" (Tyxml.Html.pp ()) output
+
+let print_monitor_inner fmt format template file transition_data collisions =
+  match format with
+  | Ben_monitor.Xhtml -> print_html_monitor fmt template file transition_data collisions
+  | _ -> Ben_monitor.print_monitor format fmt transition_data
+
+let print_monitor format template file transition_data collisions =
+  let ($) = Filename.concat in
+  let (!!) = Filename.basename in
+  let format_name = Ben_monitor.string_of_format format in
+  let htmlp = format_path_t format_name !!file in
   let html = !base $ htmlp in
   p "Generating %s\n" htmlp;
   try
-    Benl_utils.dump_xhtml_to_file html output
+    let oc = open_out html in
+    begin
+      try
+        let fmt = Format.formatter_of_out_channel oc in
+        print_monitor_inner fmt format template file transition_data collisions;
+        close_out oc
+      with e ->
+        close_out oc; Stdlib.raise e
+    end
   with e ->
-    Benl_error.warn_exn ("Failed to generate" ^ html) e
+    Error.warn_exn ("Failed to generate " ^ html) e
 
 let compute_collisions results =
   let data_map = List.fold_left
     (fun
       data_map
-      (_, transition, _, (_, transition_data, _)) ->
-        let  _, _, _, _, _, _, pkgs = transition_data in
-        let new_data = Benl_data.S.fold
+      (transition, _, transition_data) ->
+        let pkgs = transition_data.Ben_monitor.packages in
+        let new_data = Data.S.fold
           (fun package data ->
             SMap.add (Package.Name.to_string package) [transition] data
           )
@@ -311,15 +326,17 @@ let compute_collisions results =
 let generate_stats results =
   List.fold_left
     (fun (packages, profiles)
-      (p, t, export, transition_data, _) ->
-        let _, _, _, _, all, bad, pkgs = transition_data in
-        let htmlp = html_path_t t in
+      (t, p, transition_data) ->
+        let Ben_monitor.{config; all; bad; _} = transition_data in
+        let export = get_export config in
+        let pkgs = transition_data.Ben_monitor.packages in
+        let htmlp = format_path_t "html" t in
         let profiles = sadd
           profiles
           (string_of_profile p)
           (htmlp, t, all, bad)
         in
-        let packages = Benl_data.S.fold
+        let packages = Data.S.fold
           (fun package packages ->
             if export then
               sadd
@@ -362,7 +379,7 @@ let dump_yaml smap file =
     dump_to_file newfile string;
     mv newfile file
   with exn ->
-    Benl_error.error_exn ("Failed to generate " ^ file) exn
+    Error.error_exn ("Failed to generate " ^ file) exn
 
 let clean_up smap =
   if !clean then begin
@@ -409,7 +426,7 @@ let tracker template profiles =
           if all = 0
           then 0
           else 100*(all-bad)/all in
-        [ pcdata (sprintf " (%d%%)" score) ]
+        [ txt (sprintf " (%d%%)" score) ]
       else []
     )
   in
@@ -422,13 +439,13 @@ let tracker template profiles =
         with _ ->
           List.assoc Unknown profiles_desc
       in
-      let tlist = List.sort Pervasives.compare tlist in
+      let tlist = List.sort Stdlib.compare tlist in
       let tlist = List.map (tget show_score) tlist in
       match tlist with
         | [] -> acc
         | _::_ ->
           let tdiv = div ~a:[ a_class [ "transitions" ] ]
-            [ b [ pcdata title ];
+            [ b [ txt title ];
               ul tlist
             ] in
           tdiv::acc
@@ -438,26 +455,25 @@ let tracker template profiles =
   in
   let contents = template.Template.intro @ contents in
   let index = Filename.concat !base "index.html" in
-  let subtitle = [ pcdata "Transition tracker" ] in
+  let subtitle = [ txt "Transition tracker" ] in
   let output = template.Template.page ~title:page_title ~subtitle ~headers:[] ~body:contents ~footer in
   try
     p "Generating index...\n";
     dump_xhtml_to_file index output
   with exn ->
-    Benl_error.error_exn "Failed to generate index.html" exn
-
-let () = at_exit (fun () ->
-  try
-    rm [lockf ()]
-  with _ -> ()
-)
+    Error.error_exn "Failed to generate index.html" exn
 
 let main () =
-  let () = read_global_config () in
   let lockf = lockf () in
   if test Exists lockf then
     eprintf "Please wait until %s is removed!\n" lockf
   else
+    let () = at_exit (fun () ->
+      try
+        rm [lockf]
+      with _ -> ()
+      )
+    in
     try
       let lockf_b = Filename.dirname lockf in
       let () = if test Exists lockf_b then
@@ -465,15 +481,19 @@ let main () =
         else
           eprintf "%s doesn't exist. Skipping creation of lock file.\n" lockf_b
       in
-      let htmld = Filename.concat !base "html" in
-      if test (Not Exists) htmld then
-        mkdir ~parent:true htmld;
+      let () =
+        List.iter
+          (fun format ->
+            let dir = Filename.concat !base (Ben_monitor.string_of_format format) in
+            if test (Not Exists) dir then mkdir ~parent:true dir
+          ) !output_formats
+      in
       Ben_monitor.check_media_dir !base;
       let confd = !config_dir in
       let test_cond = And (Size_not_null,
                       And (Has_extension "ben",
                       And (Is_file, Is_readable))) in
-      let template = Benl_templates.get_registered_template () in
+      let template = Templates.get_registered_template () in
       (* Computing list of transitions *)
       let transition_files =
         match !tconfig with
@@ -489,17 +509,17 @@ let main () =
       in
       (* Read found .ben files *)
       let transitions =
-        Benl_parallel.map
+        Parallel.map
           (fun (transition, profile) ->
             try
               let name, config = read_transition_config transition in
               Some (name, (config, transition, profile))
             with
-            | Benl_error.Error e -> (* Ben file has errors *)
+            | Error.Error e -> (* Ben file has errors *)
               warn e;
               None
             | e ->
-              Benl_error.warn_exn ("Failed to read " ^ transition) e;
+              Error.warn_exn ("Failed to read " ^ transition) e;
               None
 
           )
@@ -515,10 +535,10 @@ let main () =
           transitions
       in
       (* Read ben.cache *)
-      let cache = read_cache (Benl_base.archs_list ()) in
+      let cache = read_cache (Clflags.archs_list ()) in
       (* Emit warnings for missing source packages *)
       let () =
-        if !Benl_clflags.verbose || not !Benl_clflags.quiet then
+        if !Clflags.verbose || not !Clflags.quiet then
           let { Marshallable.src_map = srcs; Marshallable.bin_map = bins } = cache in
           let missing = Hashtbl.create 43 in
           PAMap.iter begin fun (_, arch) pkg ->
@@ -533,21 +553,21 @@ let main () =
       in
       (* Compute data for each transition *)
       let results =
-        Benl_parallel.map
+        Parallel.map
           (fun (name, (config, _, profile)) ->
             let () =
               p "Computing data for (%s) %s\n"
                 (string_of_profile profile)
                 name
             in
-            config, name, profile, (get_transition_data cache name config)
+            name, profile, (get_transition_data cache name config)
           )
           transitions
       in
       let results =
         List.fold_left
           (fun results -> function
-          | (c, n, p, Some d) -> (c, n, p, d) :: results
+          | (n, p, Some d) -> (n, p, d) :: results
           | _ -> results
           )
           []
@@ -556,26 +576,21 @@ let main () =
       (* Compute collisions *)
       let collisions = compute_collisions results in
       (* Generate an HTML page for each transition *)
-      let () = Benl_parallel.iter
-        (fun (config, transition, _, (_, transition_data, has_testing_data)) ->
-          print_html_monitor
-            config
-            template
-            transition
-            transition_data
-            has_testing_data
-            collisions
+      let () = Parallel.iter
+        (fun (transition, _, transition_data) ->
+          List.iter
+            (fun format ->
+              print_monitor
+                format
+                template
+                transition
+                transition_data
+                collisions
+            ) !output_formats
         )
         results
       in
       (* Generate the packages.yaml file *)
-      let results =
-        Benl_parallel.map
-          (fun (_, t, p, (export, transition_data, has_testing_data)) ->
-            p, t, export, transition_data, has_testing_data
-          )
-          results
-      in
       let packages, profiles = generate_stats results in
       let () = dump_yaml packages "packages.yaml" in
       (* Clean up the HTML directory from old files *)
@@ -585,12 +600,12 @@ let main () =
         | None -> tracker template profiles
         | Some _ -> ())
     with exn ->
-      Benl_error.error_exn "General error" exn;
+      Error.error_exn "General error" exn;
       exit(42)
 
 let frontend = {
-  Benl_frontend.name = "tracker";
-  Benl_frontend.main = main;
-  Benl_frontend.anon_fun = (fun _ -> ());
-  Benl_frontend.help = spec;
+  Frontend.name = "tracker";
+  Frontend.main = main;
+  Frontend.anon_fun = (fun _ -> ());
+  Frontend.help = spec;
 }
