@@ -18,8 +18,8 @@
 (**************************************************************************)
 
 open Printf
-open Benl_types
-open Benl_core
+open Types
+open Core
 
 type 'a t = string StringMap.t
 
@@ -33,7 +33,7 @@ let filter_print keep outc p =
   StringMap.iter
     (fun f v ->
       if keep = [] || List.mem (String.lowercase_ascii f) keep then
-        fprintf outc "%s: %s\n" (Benl_core.capitalize f) v)
+        fprintf outc "%s: %s\n" (Core.capitalize f) v)
     p;
   fprintf outc "\n"
 
@@ -50,6 +50,7 @@ module Set = struct
   let is_empty = S.is_empty
   let empty = S.empty
   let add = S.add
+  let remove = S.remove
   let mem = S.mem
   let from_list =
     List.fold_left
