@@ -1,12 +1,6 @@
 open Printf
 open Tyxml.Html
 
-let href uri =
-  a_href (uri_of_string uri)
-
-let a_link url text =
-  a ~a:[a_href (uri_of_string url)] [pcdata text]
-
 let page ~title ~subtitle ~headers ~body ~footer =
   let headers =
     (meta
@@ -20,20 +14,20 @@ let page ~title ~subtitle ~headers ~body ~footer =
     headers in
   html ~a:[a_xmlns `W3_org_1999_xhtml]
     (head
-       (Tyxml.Html.title (pcdata title))
+       (Tyxml.Html.title (txt title))
        headers
     )
     (Tyxml.Html.body ~a:[a_class ["debian"]] [
-      h1 ~a:[a_id "title"] [pcdata "Transition tracker"];
+      h1 ~a:[a_id "title"] [txt "Transition tracker"];
       h2 ~a:[a_id "subtitle"] subtitle;
       div ~a:[a_id "body"] body;
       div ~a:[a_id "footer"] footer
     ])
 
-open Template
+open Ben.Template
 
 let () =
-  Benl_templates.register_template {
+  Ben_frontends.Templates.register_template {
     name = "Simple";
     page;
     intro = [];
