@@ -1,11 +1,8 @@
 open Printf
 open Tyxml.Html
 
-let href uri =
-  a_href (uri_of_string uri)
-
 let a_link url text =
-  a ~a:[a_href (uri_of_string url)] [pcdata text]
+  a ~a:[a_href (uri_of_string url)] [txt text]
 
 let page ~title ~subtitle ~headers ~body ~footer =
   let headers =
@@ -20,7 +17,7 @@ let page ~title ~subtitle ~headers ~body ~footer =
     headers in
   html ~a:[a_xmlns `W3_org_1999_xhtml]
     (head
-       (Tyxml.Html.title (pcdata title))
+       (Tyxml.Html.title (txt title))
        headers
     )
     (Tyxml.Html.body ~a:[a_class ["debian"]] [
@@ -31,10 +28,10 @@ let page ~title ~subtitle ~headers ~body ~footer =
       div ~a:[a_id "footer"] footer
     ])
 
-open Template
+open Ben.Template
 
 let () =
-  Benl_templates.register_template {
+  Ben_frontends.Templates.register_template {
     name = "Debian";
     page;
     intro = [
